@@ -6,7 +6,6 @@ class.type = "class"
 class._name = "base"
 
 class._initialized = false
-class.deferredLegacyInit = {}
 class.deferredInit = {}
 class.pageTemplate = { frames = {}, firstContentFrame = nil }
 class.defaultFrameset = {}
@@ -56,24 +55,21 @@ function class:_init (options)
       options = {}
    end
    SILE.languageSupport.loadLanguage("und") -- preload for unlocalized fallbacks
+   self:_declareBaseOptions()
    self:declareOptions()
    self:registerRawHandlers()
+   self:_declareBaseSettings()
    self:declareSettings()
+   self:_registerBaseCommands()
    self:registerCommands()
    self:setOptions(options)
    self:declareFrames(self.defaultFrameset)
    self:registerPostinit(function (self_)
-      -- In the event no packages have called \language explicitly or otherwise triggerend the language loader, at this
-      -- point we'll have a language *setting* but not actually have loaded the language. We put it off as long as we
-      -- could in case the user changed the default document language and we didn't need to load the system default one,
-      -- but that time has come at gone at this point. Make sure we've loaded somethnig...
-      local lang = SILE.settings:get("document.language")
-      SILE.languageSupport.loadLanguage(lang)
       if type(self.firstContentFrame) == "string" then
          self_.pageTemplate.firstContentFrame = self_.pageTemplate.frames[self_.firstContentFrame]
       end
       local frame = self_:initialFrame()
-      SILE.typesetter = SILE.typesetters.base(frame)
+      SILE.typesetter = SILE.typesetters.default(frame)
       SILE.typesetter:registerPageEndHook(function ()
          SU.debug("frames", function ()
             for _, v in pairs(SILE.frames) do
@@ -86,6 +82,7 @@ function class:_init (options)
 end
 
 function class:_post_init ()
+   SILE.documentState.documentClass = self
    self._initialized = true
    for i, func in ipairs(self.deferredInit) do
       func(self)
@@ -96,7 +93,7 @@ end
 
 function class:setOptions (options)
    options = options or {}
-   -- Classes that add options with dependencies should explicitly handle them, then exempt them from furthur processing.
+   -- Classes that add options with dependencies should explicitly handle them, then exempt them from further processing.
    -- The landscape and crop related options are handled explicitly before papersize, then the "rest" of options that are not interdependent.
    self.options.landscape = SU.boolean(options.landscape, false)
    options.landscape = nil
@@ -116,13 +113,15 @@ function class:declareOption (option, setter)
    self.options[option] = setter
 end
 
-function class:declareOptions ()
+function class:declareOptions () end
+
+function class:_declareBaseOptions ()
    self:declareOption("class", function (_, name)
       if name then
          if self._legacy then
             self._name = name
          elseif name ~= self._name then
-            SU.error("Cannot change class name after instantiation, derive a new class instead.")
+            SU.error("Cannot change class name after instantiation, derive a new class instead")
          end
       end
       return self._name
@@ -179,7 +178,9 @@ function class:declareOptions ()
    end)
 end
 
-function class.declareSettings (_)
+function class:declareSettings () end
+
+function class:_declareBaseSettings ()
    SILE.settings:declare({
       parameter = "current.parindent",
       type = "glue or nil",
@@ -247,10 +248,11 @@ function class:initPackage (pack, options)
       "0.14.0",
       "0.16.0",
       [[
-  This package appears to be a legacy format package. It returns a table
-  and expects SILE to guess about what to do. New packages inherit
-  from the base class and have a constructor function (_init) that
-  automatically handles setup.]]
+         This package appears to be a legacy format package. It returns a table and
+         expects SILE to guess about what to do. New packages inherit from the base
+         class and have a constructor function (_init) that automatically handles
+         setup.
+      ]]
    )
    if type(pack) == "table" then
       if pack.exports then
@@ -271,15 +273,22 @@ function class:initPackage (pack, options)
    end
 end
 
-function class:registerLegacyPostinit (func, options)
-   if self._initialized then
-      return func(self, options)
-   end
-   table.insert(self.deferredLegacyInit, function (_)
-      func(self, options)
-   end)
-end
-
+--- Register a callback function to be executed after the class initialization has completed.
+-- Sometimes a class or package may want to run things after the class has been fully initialized. This can be useful
+-- for setting document settings after packages and all their dependencies have been loaded. For example a package might
+-- want to trigger something to happen after all frames have been defined, but the package itself doesn't know if it is
+-- being loaded before or after the document options are processed, frame masters have been setup, etc. Rather than
+-- relying on the user to load the package after these events, the package can use this callback to defer the action
+-- until those things can be reasonable expected to have completed.
+--
+-- Functions in the deferred initialization queue are run on a first-set first-run basis.
+--
+-- Note the typesetter will *not* have been instantiated yet, so is not appropriate to try to output content at this
+-- point. Injecting content to be processed at the start of a document should be done with preambles. The inputter
+-- *will* be instantiated at this point, so adding a new preamble is viable.
+-- If the class has already been initialized the callback function will be run immediately.
+-- @tparam function func Callback function accepting up to two arguments.
+-- @tparam[opt] table options Additional table passed as a second argument to the callback.
 function class:registerPostinit (func, options)
    if self._initialized then
       return func(self, options)
@@ -324,7 +333,7 @@ end
 -- @tparam[opt] nil|string pack Information identifying the module registering the command for use in error and usage
 -- messages. Usually auto-detected.
 -- @see SILE.packages:registerCommand
-function class.registerCommand (_, name, func, help, pack)
+function class:registerCommand (name, func, help, pack)
    SILE.Commands[name] = func
    if not pack then
       local where = debug.getinfo(2).source
@@ -337,7 +346,7 @@ function class.registerCommand (_, name, func, help, pack)
    }
 end
 
-function class.registerRawHandler (_, format, callback)
+function class:registerRawHandler (format, callback)
    SILE.rawHandlers[format] = callback
 end
 
@@ -360,7 +369,10 @@ local function packOptions (options)
    return relevant
 end
 
-function class:registerCommands ()
+function class.registerCommands () end
+
+-- These need refactoring probably somewhere outside of the document class system
+function class:_registerBaseCommands ()
    local function replaceProcessBy (replacement, tree)
       if type(tree) ~= "table" then
          return tree
@@ -384,7 +396,11 @@ function class:registerCommands ()
          self:registerCommand(options["command"], content)
          return
       elseif options.command == "process" then
-         SU.warn("Did you mean to re-definine the `\\process` macro? That probably won't go well.")
+         SU.warn([[
+            Did you mean to re-definine the `\\process` macro?
+
+            That probably won't go well.
+         ]])
       end
       self:registerCommand(options["command"], function (_, inner_content)
          SU.debug("macros", "Processing macro \\" .. options["command"])
@@ -431,7 +447,7 @@ function class:registerCommands ()
    self:registerCommand("comment", function (_, _) end, "Ignores any text within this command's body.")
 
    self:registerCommand("process", function ()
-      SU.error("Encountered unsubstituted \\process.")
+      SU.error("Encountered unsubstituted \\process")
    end, "Within a macro definition, processes the contents of the macro body.")
 
    self:registerCommand("script", function (options, content)
@@ -443,22 +459,21 @@ function class:registerCommands ()
             "0.15.0",
             "0.16.0",
             ([[
-      The \script function has been deprecated. It was overloaded to mean
-      too many different things and more targeted tools were introduced in
-      SILE v0.14.0. To load 3rd party modules designed for use with SILE,
-      replace \script[src=...] with \use[module=...]. To run arbitrary Lua
-      code inline use \lua{}, optionally with a require= parameter to load
-      a (non-SILE) Lua module using the Lua module path or src= to load a
-      file by file path.
+               The \script function has been deprecated. It was overloaded to mean too many
+               different things and more targeted tools were introduced in SILE v0.14.0. To
+               load 3rd party modules designed for use with SILE, replace \script[src=...]
+               with \use[module=...]. To run arbitrary Lua code inline use \lua{}, optionally
+               with a require= parameter to load a (non-SILE) Lua module using the Lua module
+               path or src= to load a file by file path.
 
-      For this use case consider replacing:
+               For this use case consider replacing:
 
-          %s
+               %s
 
-      with:
+               with:
 
-          %s
-      ]]):format(original, suggested)
+               %s
+            ]]):format(original, suggested)
          )
       end
       if SU.ast.hasContent(content) then
@@ -468,7 +483,9 @@ function class:registerCommands ()
          local module = options.src:gsub("%/", ".")
          local original = (("\\script[src=%s]"):format(options.src))
          local result = SILE.require(options.src)
-         local suggested = (result._name and "\\use[module=%s]" or "\\lua[require=%s]"):format(module)
+         local suggested = (type(result) == "table" and result._name and "\\use[module=%s]" or "\\lua[require=%s]"):format(
+            module
+         )
          _deprecated(original, suggested)
          return result
       else
@@ -541,9 +558,12 @@ function class:registerCommands ()
             SILE.processString(doc, "lua", nil, packopts)
          else
             if options.src then
-               SU.warn(
-                  "Use of 'src' with \\use is discouraged because some of it's path handling\n  will eventually be deprecated. Use 'module' instead when possible."
-               )
+               SU.warn([[
+                  Use of 'src' with \\use is discouraged.
+
+                  Its path handling  will eventually be deprecated.
+                  Use 'module' instead when possible.
+               ]])
                SILE.processFile(options.src, "lua", packopts)
             else
                local module = SU.required(options, "module", "use")
